package de.vernideas.mc.taverns;

import java.util.Collection;

import net.minecraftforge.common.Configuration;
import cpw.mods.fml.common.Loader;
import cpw.mods.fml.common.registry.VillagerRegistry;

public class Settings
{
	private static final String CATEGORY_VILLAGERS = "villagers";
    // Settings
    public final boolean generateTaverns;
    public final boolean generateStalls;
    public final boolean generateBarns;
    
    public final int weightTaverns;
    public final int weightStalls;
    public final int weightBarns;
    
    public final int barWenchID;
    public final int hostlerID;
    public final int shepherdessID;
    
    // Installed mods
    public final boolean minecraftComesAlive;
    public final boolean biomesOPlenty;
    public final boolean grimoireOfGaia2;
    public final boolean millenaire;
    
    private int getVillagerID(Configuration cfg, String category, String configName, int defaultID)
    {
		Collection<Integer> registeredVillagers = VillagerRegistry.getRegisteredVillagers();
		while( registeredVillagers.contains(Integer.valueOf(defaultID)) )
		{
			++ defaultID;
		}
		return cfg.get(category, configName, defaultID).getInt(defaultID);
    }
    
    public Settings(Configuration config)
	{
		config.load();
		
		config.getCategory(Configuration.CATEGORY_GENERAL).setComment(
				"Enable or disable village parts as needed.\n\n"
				+ "\"Weight\" determines how likely this structure is to spawn,\n"
				+ "useful values are in the range 1-50.\n"
				+ "A higher value means the structure is more likely to be created.\n\n"
				+ "Disabling a structure disables the corresponding custom villager.");
		config.getCategory(CATEGORY_VILLAGERS).setComment(
				"IDs of custom villagers. Set it to -1 to disable\n"
				+ "this specific villager type.");
		
		generateTaverns = config.get(Configuration.CATEGORY_GENERAL, "generateTaverns", true).getBoolean(true);
		generateStalls = config.get(Configuration.CATEGORY_GENERAL, "generateStalls", true).getBoolean(true);
		generateBarns = config.get(Configuration.CATEGORY_GENERAL, "generateBarns", true).getBoolean(true);
		
		weightTaverns = config.get(Configuration.CATEGORY_GENERAL, "weightTaverns", 4).getInt(4);
		weightStalls = config.get(Configuration.CATEGORY_GENERAL, "weightStalls", 8).getInt(8);
		weightBarns = config.get(Configuration.CATEGORY_GENERAL, "weightBarns", 10).getInt(10);
		
		barWenchID = getVillagerID(config, CATEGORY_VILLAGERS, "villagerIDBarWench", 42);
		hostlerID = getVillagerID(config, CATEGORY_VILLAGERS, "villagerIDHostler", 43);
		shepherdessID = getVillagerID(config, CATEGORY_VILLAGERS, "villagerIDShepherdess", 44);

        if( config.hasChanged() )
        {
            config.save();
        }
        
    	minecraftComesAlive = Loader.isModLoaded("MCA");
    	biomesOPlenty = Loader.isModLoaded("BiomesOPlenty");
    	grimoireOfGaia2 = Loader.isModLoaded("GrimoireGaia2");
    	millenaire = Loader.isModLoaded("millenaire");
	}
}
