package de.vernideas.mc.taverns.direction;
import java.util.HashMap;
import java.util.Map;

import net.minecraft.block.Block;
import net.minecraftforge.common.ForgeDirection;

import org.apache.commons.lang3.tuple.ImmutablePair;
import org.apache.commons.lang3.tuple.Pair;

public class Dir
{
	public final static Map<Pair<Block, ForgeDirection>, Integer> map;

	public final static Map<Pair<ForgeDirection, Integer>, ForgeDirection> orientation;
	
	public static int getMeta(Block bl, ForgeDirection dir)
	{
		Integer val = map.get(ImmutablePair.<Block, ForgeDirection>of(bl, dir));
		return (null != val ? val.intValue() : dir.ordinal() );
	}
	
	public static int getMeta(Block bl, ForgeDirection dir, int orient)
	{
		ForgeDirection newDir = orientation.get(ImmutablePair.<ForgeDirection, Integer>of(dir, Integer.valueOf(orient)));
		return (null != newDir ? getMeta(bl, newDir) : getMeta(bl, dir));
	}
	
	private static void add(Block bl, ForgeDirection dir, int val)
	{
		map.put(ImmutablePair.<Block, ForgeDirection>of(bl, dir), Integer.valueOf(val));
	}
	
	private static void addCardinal(Block bl, int valNorth, int valSouth, int valWest, int valEast)
	{
		map.put(ImmutablePair.<Block, ForgeDirection>of(bl, ForgeDirection.NORTH), Integer.valueOf(valNorth));
		map.put(ImmutablePair.<Block, ForgeDirection>of(bl, ForgeDirection.SOUTH), Integer.valueOf(valSouth));
		map.put(ImmutablePair.<Block, ForgeDirection>of(bl, ForgeDirection.WEST), Integer.valueOf(valWest));
		map.put(ImmutablePair.<Block, ForgeDirection>of(bl, ForgeDirection.EAST), Integer.valueOf(valEast));
	}
	
	static
	{
		map = new HashMap<Pair<Block,ForgeDirection>, Integer>();
		
		add(Block.wood, ForgeDirection.UP, 0);
		add(Block.wood, ForgeDirection.DOWN, 0);
		addCardinal(Block.wood, 8, 8, 4, 4);
		
		add(Block.hay, ForgeDirection.UP, 0);
		add(Block.hay, ForgeDirection.DOWN, 0);
		addCardinal(Block.hay, 8, 8, 4, 4);

		addCardinal(Block.bed, 2, 0, 1, 3);

		add(Block.pistonBase, ForgeDirection.UP, 1);
		add(Block.pistonBase, ForgeDirection.DOWN, 0);
		addCardinal(Block.pistonBase, 2, 3, 4, 5);
		add(Block.pistonStickyBase, ForgeDirection.UP, 1);
		add(Block.pistonStickyBase, ForgeDirection.DOWN, 0);
		addCardinal(Block.pistonStickyBase, 2, 3, 4, 5);
		add(Block.pistonExtension, ForgeDirection.UP, 1);
		add(Block.pistonExtension, ForgeDirection.DOWN, 0);
		addCardinal(Block.pistonExtension, 2, 3, 4, 5);
		
		addCardinal(Block.stairsBrick, 3, 2, 1, 0);
		addCardinal(Block.stairsCobblestone, 3, 2, 1, 0);
		addCardinal(Block.stairsNetherBrick, 3, 2, 1, 0);
		addCardinal(Block.stairsSandStone, 3, 2, 1, 0);
		addCardinal(Block.stairsStoneBrick, 3, 2, 1, 0);
		addCardinal(Block.stairsWoodBirch, 3, 2, 1, 0);
		addCardinal(Block.stairsWoodJungle, 3, 2, 1, 0);
		addCardinal(Block.stairsWoodOak, 3, 2, 1, 0);
		addCardinal(Block.stairsWoodSpruce, 3, 2, 1, 0);
		
		addCardinal(Block.doorIron, 1, 3, 0, 2);
		addCardinal(Block.doorWood, 1, 3, 0, 2);
		
		addCardinal(Block.fenceGate, 2, 0, 1, 3);
		
		addCardinal(Block.tripWireSource, 2, 0, 1, 3);
		
		addCardinal(Block.ladder, 2, 3, 4, 5);
		addCardinal(Block.signWall, 2, 3, 4, 5);
		addCardinal(Block.furnaceIdle, 2, 3, 4, 5);
		addCardinal(Block.furnaceBurning, 2, 3, 4, 5);
		addCardinal(Block.chest, 2, 3, 4, 5);
		
		addCardinal(Block.trapdoor, 1, 0, 3, 2);
		
		add(Block.torchWood, ForgeDirection.DOWN, 5);
		addCardinal(Block.torchWood, 4, 3, 2, 1);
		add(Block.torchRedstoneIdle, ForgeDirection.DOWN, 5);
		addCardinal(Block.torchRedstoneIdle, 4, 3, 2, 1);
		add(Block.torchRedstoneActive, ForgeDirection.DOWN, 5);
		addCardinal(Block.torchRedstoneActive, 4, 3, 2, 1);
		
		orientation = new HashMap<Pair<ForgeDirection,Integer>, ForgeDirection>();
		// Orientation 0 - reflection; switch north and south
		orientation.put(ImmutablePair.<ForgeDirection, Integer>of(ForgeDirection.NORTH, Integer.valueOf(0)), ForgeDirection.SOUTH);
		orientation.put(ImmutablePair.<ForgeDirection, Integer>of(ForgeDirection.SOUTH, Integer.valueOf(0)), ForgeDirection.NORTH);
		orientation.put(ImmutablePair.<ForgeDirection, Integer>of(ForgeDirection.WEST, Integer.valueOf(0)), ForgeDirection.WEST);
		orientation.put(ImmutablePair.<ForgeDirection, Integer>of(ForgeDirection.EAST, Integer.valueOf(0)), ForgeDirection.EAST);
		// Orientation 1 - rotation?
		orientation.put(ImmutablePair.<ForgeDirection, Integer>of(ForgeDirection.NORTH, Integer.valueOf(1)), ForgeDirection.WEST);
		orientation.put(ImmutablePair.<ForgeDirection, Integer>of(ForgeDirection.SOUTH, Integer.valueOf(1)), ForgeDirection.EAST);
		orientation.put(ImmutablePair.<ForgeDirection, Integer>of(ForgeDirection.WEST, Integer.valueOf(1)), ForgeDirection.NORTH);
		orientation.put(ImmutablePair.<ForgeDirection, Integer>of(ForgeDirection.EAST, Integer.valueOf(1)), ForgeDirection.SOUTH);
		// Orientation 2 - no change
		orientation.put(ImmutablePair.<ForgeDirection, Integer>of(ForgeDirection.NORTH, Integer.valueOf(2)), ForgeDirection.NORTH);
		orientation.put(ImmutablePair.<ForgeDirection, Integer>of(ForgeDirection.SOUTH, Integer.valueOf(2)), ForgeDirection.SOUTH);
		orientation.put(ImmutablePair.<ForgeDirection, Integer>of(ForgeDirection.WEST, Integer.valueOf(2)), ForgeDirection.WEST);
		orientation.put(ImmutablePair.<ForgeDirection, Integer>of(ForgeDirection.EAST, Integer.valueOf(2)), ForgeDirection.EAST);
		// Orientation 3 - like 2
		orientation.put(ImmutablePair.<ForgeDirection, Integer>of(ForgeDirection.NORTH, Integer.valueOf(3)), ForgeDirection.EAST);
		orientation.put(ImmutablePair.<ForgeDirection, Integer>of(ForgeDirection.SOUTH, Integer.valueOf(3)), ForgeDirection.WEST);
		orientation.put(ImmutablePair.<ForgeDirection, Integer>of(ForgeDirection.WEST, Integer.valueOf(3)), ForgeDirection.NORTH);
		orientation.put(ImmutablePair.<ForgeDirection, Integer>of(ForgeDirection.EAST, Integer.valueOf(3)), ForgeDirection.SOUTH);
	}
}
