package de.vernideas.mc.taverns.gen;

import java.util.List;
import java.util.Random;

import net.minecraft.block.Block;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.passive.EntitySheep;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.world.World;
import net.minecraft.world.gen.structure.ComponentVillageStartPiece;
import net.minecraft.world.gen.structure.MapGenStructureIO;
import net.minecraft.world.gen.structure.StructureBoundingBox;
import net.minecraft.world.gen.structure.StructureComponent;
import net.minecraftforge.common.ForgeDirection;
import cpw.mods.fml.common.FMLLog;
import de.vernideas.mc.taverns.Settings;
import de.vernideas.mc.taverns.Taverns;

public class ComponentVillageBarn extends ComponentVillageBase
{
    public static boolean initialized = false;;
    
    public static void init(Settings config)
    {
    	if( !initialized )
    	{
    		initialized = true;
            MapGenStructureIO.func_143031_a(ComponentVillageBarn.class, "taverns:ViB");
    	}
    }

    private static final int HEIGHT = 6;

    public static Object buildComponent(ComponentVillageStartPiece startPiece, List pieces, Random random,
    		int x, int y, int z, int direction, int type)
    {
    	boolean extendedBuilding = (random.nextInt(3) == 0);
    	StructureBoundingBox structBB = StructureBoundingBox.getComponentToAddBoundingBox(x, y, z, 0, 0, 0, (extendedBuilding ? 13 : 9), HEIGHT, 7, direction);
    	if( canVillageGoDeeper(structBB) )
    	{
            if( null == StructureComponent.findIntersecting(pieces, structBB) )
            {
                    return new ComponentVillageBarn(startPiece, type, random, structBB, direction, extendedBuilding);
            }
    	}
    	return null;
    }
    
    private int averageGroundLevel = -1;
    
    private boolean hasExtension = false;
    private int woolColor = 0;
    private int sheepSpawned = 0;

    public ComponentVillageBarn() {}
    
	public ComponentVillageBarn(ComponentVillageStartPiece startPiece, int pieces, Random rnd, StructureBoundingBox structBB, int direction, boolean extension)
	{
		super(startPiece, pieces);
        coordBaseMode = direction;
        boundingBox = structBB;
        hasExtension = extension;
        woolColor = rnd.nextInt(16);
        sheepSpawned = rnd.nextInt(2); // Half of the time just one sheep
    }

	/** writeNBTTag */
    @Override
    protected void func_143012_a(NBTTagCompound par1NBTTagCompound)
    {
        super.func_143012_a(par1NBTTagCompound);
        par1NBTTagCompound.setBoolean("HasExtension", hasExtension);
        par1NBTTagCompound.setInteger("WoolC", woolColor);
        par1NBTTagCompound.setInteger("VSheep", sheepSpawned);
    }

	/** readNBTTag */
    @Override
    protected void func_143011_b(NBTTagCompound par1NBTTagCompound)
    {
        super.func_143011_b(par1NBTTagCompound);
        hasExtension = par1NBTTagCompound.getBoolean("HasExtension");
        woolColor = par1NBTTagCompound.getInteger("WoolC");
        sheepSpawned = par1NBTTagCompound.getInteger("VSheep");
    }

	@Override
	public boolean addComponentParts(World world, Random random, StructureBoundingBox bb)
	{
        if( averageGroundLevel < 0 )
        {
            averageGroundLevel = getAverageGroundLevel(world, bb);
            if (averageGroundLevel < 0)
            {
                return true;
            }
            boundingBox.offset(0, this.averageGroundLevel - boundingBox.maxY + HEIGHT - 2, 0);
        }
        
        int width = (hasExtension ? 13 : 9);

        // Clear area for the barn building
        fillWithBlocks(world, bb, 0,0,0, width-1,5,6, 0, 0, false);
        for( int xx = 0; xx < width; ++ xx )
        {
            for( int zz = 0; zz < 7; ++ zz)
            {
                clearCurrentPositionBlocksUpwards(world, xx,0,zz, bb);
                fillDownwards(world, Block.cobblestone, 0, xx,-1,zz, bb);
            }
        }

        // Ground
        fill(world, bb, 0,0,0, width-1,0,6, Block.grass, Block.grass); // Floor
        fill(world, bb, 3,0,0, 5,0,0, Block.gravel, Block.gravel); // Entrance
        
        // Torches
        placeTorch(world, bb, 3,2,1, ForgeDirection.DOWN);
        placeTorch(world, bb, 5,2,1, ForgeDirection.DOWN);
        placeTorch(world, bb, 0,2,5, ForgeDirection.WEST);
        placeTorch(world, bb, 8,2,5, ForgeDirection.EAST);
        placeTorch(world, bb, 1,2,6, ForgeDirection.NORTH);
        placeTorch(world, bb, 7,2,6, ForgeDirection.NORTH);
        
        // Enclosure
        fill(world, bb, 1,1,1, 7,1,1, Block.fence, Block.fence);
        fill(world, bb, 2,1,5, 6,1,5, Block.fence, Block.fence);
        fill(world, bb, 1,1,2, 1,1,4, Block.fence, Block.fence);
        fill(world, bb, 7,1,2, 7,1,4, Block.fence, Block.fence);
        placeBlock(world, Block.fenceGate, ForgeDirection.SOUTH, 4,1,1, bb);
        placeBlock(world, Block.wood, 0, 1,1,5, bb);
        placeBlock(world, Block.wood, 0, 7,1,5, bb);

        placeBlock(world, Block.fence, 0, 1,2,1, bb);
        placeBlock(world, Block.fence, 0, 1,3,1, bb);
        placeBlock(world, Block.fence, 0, 7,2,1, bb);
        placeBlock(world, Block.fence, 0, 7,3,1, bb);
        placeBlock(world, Block.woodDoubleSlab, 0, 1,2,5, bb);
        placeBlock(world, Block.woodDoubleSlab, 0, 7,2,5, bb);
        
        // Roofing
        placeBlock(world, Block.woodSingleSlab, 0, 1,4,1, bb);
        placeBlock(world, Block.cloth, 0, 2,4,1, bb);
        placeBlock(world, Block.cloth, woolColor, 3,4,1, bb);
        placeBlock(world, Block.cloth, 0, 4,4,1, bb);
        placeBlock(world, Block.cloth, woolColor, 5,4,1, bb);
        placeBlock(world, Block.cloth, 0, 6,4,1, bb);
        placeBlock(world, Block.woodSingleSlab, 0, 7,4,1, bb);
        
        for( int xx = 1; xx <= 7; ++ xx )
        {
        	placeBlock(world, Block.woodSingleSlab, 0, xx,4,2, bb);
        	placeBlock(world, Block.woodSingleSlab, 8, xx,3,3, bb);
        	placeBlock(world, Block.woodSingleSlab, 0, xx,3,4, bb);
        }

        for( int xx = 2; xx <= 6; ++ xx )
        {
        	placeBlock(world, Block.woodSingleSlab, 8, xx,2,5, bb);
        }
        
        sheepSpawned += spawnEntity(world, bb, 3,1,3, 1 - sheepSpawned);
        sheepSpawned += spawnEntity(world, bb, 5,1,3, 2 - sheepSpawned);

        if( hasExtension )
        {
        	// Added building to the side
        	
        	// Light
        	placeTorch(world, bb, 9,4,0, ForgeDirection.SOUTH);
        	placeTorch(world, bb, 8,4,3, ForgeDirection.EAST);

        	// Ground
        	fill(world, bb, 7,1,1, 11,1,5, Block.cobblestone, Block.cobblestone);
        	placeBlock(world, Block.stoneSingleSlab, 3, 7,1,3, bb);
        	placeBlock(world, Block.stairsCobblestone, ForgeDirection.NORTH, 9,1,0, bb);
        	placeBlock(world, Block.stairsCobblestone, ForgeDirection.EAST, 8,1,0, bb);
        	placeBlock(world, Block.stairsCobblestone, ForgeDirection.WEST, 10,1,0, bb);
        	
        	// Walls
        	for( int yy = 2; yy <= 4; ++ yy )
        	{
        		placeBlock(world, Block.wood, 0, 7,yy,1, bb);
        		placeBlock(world, Block.planks, 0, 8,yy,1, bb);
        		placeBlock(world, Block.planks, 0, 10,yy,1, bb);
        		placeBlock(world, Block.wood, 0, 11,yy,1, bb);
        		placeBlock(world, Block.wood, 0, 7,yy,5, bb);
        		placeBlock(world, Block.wood, 0, 11,yy,5, bb);
        		placeBlock(world, Block.planks, 0, 7,yy,2, bb);
        		placeBlock(world, Block.planks, 0, 7,yy,4, bb);
        	}
        	fill(world, bb, 8,2,5, 10,4,5, Block.planks, Block.planks);
        	fill(world, bb, 11,2,2, 11,4,4, Block.planks, Block.planks);
        	placeBlock(world, Block.planks, 0, 9,4,1, bb);
        	placeBlock(world, Block.planks, 0, 7,4,3, bb);
        	placeAir(world, 7,3,3, bb);
        	for( int xx = 8; xx <= 10; ++ xx )
        	{
        		placeBlock(world, Block.wood, 0, xx,5,1, bb);
        		placeBlock(world, Block.wood, 0, xx,5,5, bb);
        	}
        	
        	placeBlock(world, Block.thinGlass, 0, 9,3,5, bb);
        	placeBlock(world, Block.thinGlass, 0, 11,3,3, bb);
        	placeDoor(world, bb, random, 9,2,1, ForgeDirection.SOUTH);
        	
        	// Roofing
        	for( int zz = 1; zz <= 5; ++ zz )
        	{
        		placeBlock(world, Block.stairsWoodOak, ForgeDirection.EAST, 7,5,zz, bb);
        		placeBlock(world, Block.stairsWoodOak, ForgeDirection.WEST, 11,5,zz, bb);
        		placeBlock(world, Block.woodSingleSlab, 0, 8,6,zz, bb);
        		placeBlock(world, Block.planks, 0, 9,6,zz, bb);
        		placeBlock(world, Block.woodSingleSlab, 0, 10,6,zz, bb);
        	}
        	
            spawnVillagers(world, bb, 9,2,3, 1);
        }
        
        return true;
	}

	@Override
	protected int getVillagerType(int alreadySpawned)
	{
		return (Taverns.config.shepherdessID > 0 ? Taverns.config.shepherdessID : super.getVillagerType(alreadySpawned));
	}
	
	@Override
	protected EntityLivingBase getEntity(World world, int index)
	{
		return new EntitySheep(world);
	}
}
