package de.vernideas.mc.taverns.gen;

import java.util.Random;

import de.vernideas.mc.taverns.direction.Dir;
import net.minecraft.block.Block;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.passive.EntityVillager;
import net.minecraft.world.World;
import net.minecraft.world.gen.structure.ComponentVillage;
import net.minecraft.world.gen.structure.ComponentVillageStartPiece;
import net.minecraft.world.gen.structure.StructureBoundingBox;
import net.minecraftforge.common.ForgeDirection;

public abstract class ComponentVillageBase extends ComponentVillage
{
	public ComponentVillageBase() {}
	
	protected ComponentVillageBase(ComponentVillageStartPiece startPiece, int orient)
	{
		super(startPiece, orient);
	}

	// Helper methods for less writing and preparation for 1.7 changes
	protected void fill(World worldObj, StructureBoundingBox structBB,
			int minX, int minY, int minZ, int maxX, int maxY, int maxZ,
			Block placeBlock, Block replaceBlock, boolean alwaysreplace)
	{
		fillWithBlocks(worldObj, structBB,
				minX, minY, minZ, maxX, maxY, maxZ,
				placeBlock.blockID, replaceBlock.blockID, alwaysreplace);
	}

	protected void fill(World worldObj, StructureBoundingBox structBB,
			int minX, int minY, int minZ, int maxX, int maxY, int maxZ,
			Block placeBlock, Block replaceBlock)
	{
		fillWithBlocks(worldObj, structBB,
				minX, minY, minZ, maxX, maxY, maxZ,
				placeBlock.blockID, replaceBlock.blockID, false);
	}

	protected void fillRandomly(World world, StructureBoundingBox structBB, Random rnd, float chance,
			int minX, int minY, int minZ, int maxX, int maxY, int maxZ,
			Block placeBlock, Block replaceBlock)
	{
		randomlyFillWithBlocks(world, structBB, rnd, chance,
				minX, minY, minZ, maxX, maxY, maxZ,
				placeBlock.blockID, replaceBlock.blockID, false);
	}
	
	protected void fillDownwards(World world,
			Block block, int par3, int xx, int par5, int zz,
			StructureBoundingBox structBB)
	{
		fillCurrentPositionBlocksDownwards(world, block.blockID, par3, xx, par5, zz, structBB);
	}

	protected void placeBlock(World world, Block block,
			int metadata, int posX, int posY, int posZ,
			StructureBoundingBox structBB)
	{
		placeBlockAtCurrentPosition(world, block.blockID, metadata, posX, posY, posZ, structBB);
	}
	
	protected void placeBlock(World world, Block block, ForgeDirection dir,
			int posX, int posY, int posZ, StructureBoundingBox structBB)
	{
		placeBlockAtCurrentPosition(world, block.blockID, Dir.getMeta(block, dir, coordBaseMode), posX, posY, posZ, structBB);
	}
	
	protected void placeAir(World world, int posX, int posY, int posZ, StructureBoundingBox structBB)
	{
		placeBlockAtCurrentPosition(world, 0, 0, posX, posY, posZ, structBB);
	}

	protected void placeDoor(World world, StructureBoundingBox structBB,
			Random rnd, int posX, int posY, int posZ, ForgeDirection dir)
	{
		placeDoorAtCurrentPosition(world, structBB, rnd, posX, posY, posZ, Dir.getMeta(Block.doorWood, dir, coordBaseMode));
	}
	
	/**
	 * Warning: Sets the position *below* this one to air!
	 */
	protected void placeTorch(World world, StructureBoundingBox structBB,
			int posX, int posY, int posZ, ForgeDirection dir)
	{
        placeBlock(world, Block.stone, 0, posX, posY - 1, posZ, structBB);
        placeBlock(world, Block.torchWood, dir, posX, posY, posZ, structBB);
        placeAir(world, posX, posY - 1, posZ, structBB);
	}
	
	protected void setMetadata(World world, int metadata,
			int posX, int posY, int posZ, StructureBoundingBox structBB)
	{
		int globalX = getXWithOffset(posX, posZ);
		int globalY = getYWithOffset(posY);
		int globalZ = getZWithOffset(posX, posZ);
		
		if( structBB.isVecInside(globalX, globalY, globalZ) )
		{
			world.setBlockMetadataWithNotify(globalX, globalY, globalZ, metadata, 2);
		}
	}
	
	/**
	 * Does nothing if amount <= 0.
	 * Else tries to spawn entities by calling getEntity(index) as often as needed.
	 * 
	 * @return Amount of entities spawned
	 */
    protected int spawnEntity(World world, StructureBoundingBox structBB, int posX, int posY, int posZ, int amount)
    {
    	int spawned = 0;
    	for( int idx = 0; idx < amount; ++ idx )
    	{
    		int globalX = getXWithOffset(posX, posY);
    		int globalY = getYWithOffset(posY);
    		int globalZ = getZWithOffset(posX, posZ);
    		
    		if( structBB.isVecInside(globalX, globalY, globalZ) )
    		{
    			EntityLivingBase entity = getEntity(world, idx);
    			if( null != entity )
    			{
    				++ spawned;
    				entity.setLocationAndAngles((double)globalX + 0.5d, (double)globalY, (double)globalZ + 0.5d, 0.5f, 0.0f);
    				world.spawnEntityInWorld(entity);
    			}
    		}
    	}
    	
    	return spawned;
    }
    
    /**
     * Override this method for custom (non-villager) entity spawning.
     * 
     * @param world the World to spawn the entity in
     * @param index the index number of the entity
     * @return the EntityLivingBase to spawn, or null if we don't spawn any
     */
    protected EntityLivingBase getEntity(World world, int index)
    {
    	return null;
    }

}
