package de.vernideas.mc.taverns.gen;

import java.util.List;
import java.util.Random;

import net.minecraft.block.Block;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.passive.EntityHorse;
import net.minecraft.item.Item;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.WeightedRandomChestContent;
import net.minecraft.world.World;
import net.minecraft.world.gen.structure.ComponentVillageStartPiece;
import net.minecraft.world.gen.structure.MapGenStructureIO;
import net.minecraft.world.gen.structure.StructureBoundingBox;
import net.minecraft.world.gen.structure.StructureComponent;
import net.minecraftforge.common.ChestGenHooks;
import net.minecraftforge.common.ForgeDirection;
import de.vernideas.mc.taverns.Settings;
import de.vernideas.mc.taverns.Taverns;
import de.vernideas.mc.taverns.direction.Dir;

public class ComponentVillageStall extends ComponentVillageBase
{
    public static final String STALL_CHEST = "taverns:stallChest";
    public static final WeightedRandomChestContent[] stallChestContents = new WeightedRandomChestContent[]
    {
        new WeightedRandomChestContent(Item.diamond.itemID, 0, 1, 1, 3),
        new WeightedRandomChestContent(Item.appleGold.itemID, 1, 1, 1, 1),
        new WeightedRandomChestContent(Item.appleGold.itemID, 0, 1, 1, 3),
        new WeightedRandomChestContent(Item.horseArmorIron.itemID, 0, 1, 1, 5),
        new WeightedRandomChestContent(Item.horseArmorGold.itemID, 0, 1, 1, 2),
        new WeightedRandomChestContent(Item.saddle.itemID, 0, 1, 1, 40),
        new WeightedRandomChestContent(Item.goldenCarrot.itemID, 0, 1, 3, 10),
        new WeightedRandomChestContent(Item.appleRed.itemID, 0, 1, 8, 50),
        new WeightedRandomChestContent(Block.pumpkin.blockID, 0, 1, 3, 25),
        new WeightedRandomChestContent(Block.hay.blockID, 0, 1, 5, 30),
        new WeightedRandomChestContent(Item.carrot.itemID, 0, 1, 5, 30),
        new WeightedRandomChestContent(Item.leather.itemID, 0, 1, 9, 30),
        new WeightedRandomChestContent(Item.nameTag.itemID, 0, 1, 1, 10),
    };

    public static boolean initialized = false;;
    
    public static void init(Settings config)
    {
    	if( !initialized )
    	{
    		initialized = true;
    		ChestGenHooks.getInfo(STALL_CHEST).setMin(1);
    		ChestGenHooks.getInfo(STALL_CHEST).setMax(10);
            for( int i = 0; i < stallChestContents.length; ++ i )
            {
            	ChestGenHooks.addItem(STALL_CHEST, stallChestContents[i]);
            }
            MapGenStructureIO.func_143031_a(ComponentVillageStall.class, "taverns:ViS");
    	}
    }

    private static final int HEIGHT = 7;

    public static Object buildComponent(ComponentVillageStartPiece startPiece, List pieces, Random random,
    		int x, int y, int z, int direction, int type)
    {
    	StructureBoundingBox structBB = StructureBoundingBox.getComponentToAddBoundingBox(x, y, z, 0, 0, 0, 11, HEIGHT, 10, direction);
    	if( canVillageGoDeeper(structBB) )
    	{
            if( null == StructureComponent.findIntersecting(pieces, structBB) )
            {
                    return new ComponentVillageStall(startPiece, type, random, structBB, direction);
            }
    	}
    	return null;
    }
    
    private int averageGroundLevel = -1;
    private boolean horseSpawned = false;
    private boolean hasMadeChest = false;

    public ComponentVillageStall() {}
    
    public ComponentVillageStall(ComponentVillageStartPiece startPiece, int pieces, Random rnd, StructureBoundingBox structBB, int direction)
	{
		super(startPiece, pieces);
        coordBaseMode = direction;
        boundingBox = structBB;
    }

	/** writeNBTTag */
    @Override
    protected void func_143012_a(NBTTagCompound par1NBTTagCompound)
    {
        super.func_143012_a(par1NBTTagCompound);
        par1NBTTagCompound.setBoolean("HorseSpawned", horseSpawned);
        par1NBTTagCompound.setBoolean("Chest", hasMadeChest);
    }

	/** readNBTTag */
    @Override
    protected void func_143011_b(NBTTagCompound par1NBTTagCompound)
    {
        super.func_143011_b(par1NBTTagCompound);
        horseSpawned = par1NBTTagCompound.getBoolean("HorseSpawned");
        hasMadeChest = par1NBTTagCompound.getBoolean("Chest");
    }

	@Override
	public boolean addComponentParts(World world, Random random, StructureBoundingBox bb)
	{
        if( averageGroundLevel < 0 )
        {
            averageGroundLevel = getAverageGroundLevel(world, bb);
            if (averageGroundLevel < 0)
            {
                return true;
            }
            boundingBox.offset(0, this.averageGroundLevel - boundingBox.maxY + HEIGHT - 2, 0);
        }
        
        // Clear area for the stall building
        fillWithBlocks(world, bb, 0,0,0, 10,6,9, 0, 0, false);
        for( int xx = 0; xx < 11; ++ xx )
        {
            for( int zz = 0; zz < 10; ++ zz)
            {
                clearCurrentPositionBlocksUpwards(world, xx,0,zz, bb);
                fillDownwards(world, Block.cobblestone, 0, xx,-1,zz, bb);
            }
        }
        
        // Ground
        fill(world, bb, 0,0,0, 10,0,9, Block.grass, Block.grass); // Grass around the place
        fill(world, bb, 2,0,2, 8,0,7, Block.cobblestone, Block.cobblestone); // Floor
        fillRandomly(world, bb, random, 0.2f, 2,0,2, 8,0,7, Block.cobblestoneMossy, Block.cobblestoneMossy);
        fill(world, bb, 4,0,0, 6,0,1, Block.gravel, Block.gravel); // Entrance

        // Torches
        placeTorch(world, bb, 2,3,2, ForgeDirection.SOUTH);
        placeTorch(world, bb, 5,3,2, ForgeDirection.SOUTH);
        placeTorch(world, bb, 8,3,2, ForgeDirection.SOUTH);
        placeTorch(world, bb, 3,4,5, ForgeDirection.EAST);
        placeTorch(world, bb, 7,4,5, ForgeDirection.WEST);
        
        // Wood structure
        for( int yy = 1; yy <= 3; ++ yy )
        {
        	placeBlock(world, Block.wood, 0, 2,yy,3, bb);
        	placeBlock(world, Block.wood, 0, 5,yy,3, bb);
        	placeBlock(world, Block.wood, 0, 8,yy,3, bb);
        	placeBlock(world, Block.wood, 0, 2,yy,7, bb);
        	placeBlock(world, Block.wood, 0, 5,yy,7, bb);
        	placeBlock(world, Block.wood, 0, 8,yy,7, bb);
        }
        for( int xx = 2; xx <= 8; ++ xx )
        {
        	placeBlock(world, Block.wood, ForgeDirection.EAST, xx,4,3, bb);
        	placeBlock(world, Block.wood, ForgeDirection.EAST, xx,6,5, bb);
        	placeBlock(world, Block.wood, ForgeDirection.EAST, xx,4,7, bb);
        }
        placeBlock(world, Block.wood, ForgeDirection.EAST, 1,6,5, bb);
        placeBlock(world, Block.wood, ForgeDirection.EAST, 9,6,5, bb);
        for( int zz = 4; zz <= 6; ++ zz )
        {
        	placeBlock(world, Block.wood, ForgeDirection.NORTH, 2,4,zz, bb);
        	placeBlock(world, Block.wood, ForgeDirection.NORTH, 5,5,zz, bb);
        	placeBlock(world, Block.wood, ForgeDirection.NORTH, 8,4,zz, bb);
        }
        
        // Walls
        for( int xx = 2; xx <= 8; xx += 6 )
        {
	        placeBlock(world, Block.cobblestone, 0, xx,1,4, bb);
	        placeBlock(world, Block.planks, 0, xx,1,5, bb);
	        placeBlock(world, Block.cobblestone, 0, xx,1,6, bb);

	        placeBlock(world, Block.planks, 0, xx,2,4, bb);
	        placeBlock(world, Block.cobblestone, 0, xx,2,5, bb);
	        placeBlock(world, Block.planks, 0, xx,2,6, bb);

	        placeBlock(world, Block.cobblestone, 0, xx,3,4, bb);
	        placeBlock(world, Block.planks, 0, xx,3,5, bb);
	        placeBlock(world, Block.cobblestone, 0, xx,3,6, bb);

	        placeBlock(world, Block.cobblestone, 0, xx,5,4, bb);
	        placeBlock(world, Block.stoneSingleSlab, 11, xx,5,5, bb);
	        placeBlock(world, Block.cobblestone, 0, xx,5,6, bb);
        }
        placeBlock(world, Block.cobblestone, 0, 3,1,7, bb);
        placeBlock(world, Block.cobblestone, 0, 4,1,7, bb);
        placeBlock(world, Block.cobblestone, 0, 6,1,7, bb);
        placeBlock(world, Block.cobblestone, 0, 7,1,7, bb);
        placeBlock(world, Block.cobblestoneWall, 0, 3,2,7, bb);
        placeBlock(world, Block.cobblestoneWall, 0, 4,2,7, bb);
        placeBlock(world, Block.cobblestoneWall, 0, 6,2,7, bb);
        placeBlock(world, Block.cobblestoneWall, 0, 7,2,7, bb);

        // Fences
        placeBlock(world, Block.fence, 0, 5,1,4, bb);
        placeBlock(world, Block.fence, 0, 5,1,5, bb);
        placeBlock(world, Block.fence, 0, 5,1,6, bb);
        placeBlock(world, Block.fence, 0, 5,3,4, bb);
        placeBlock(world, Block.fence, 0, 5,3,6, bb);
        placeBlock(world, Block.fenceGate, ForgeDirection.NORTH, 3,1,3, bb);
        placeBlock(world, Block.fenceGate, ForgeDirection.NORTH, 4,1,3, bb);
        placeBlock(world, Block.fenceGate, ForgeDirection.NORTH, 6,1,3, bb);
        placeBlock(world, Block.fenceGate, ForgeDirection.NORTH, 7,1,3, bb);

        // Roofing
        for( int xx = 1; xx <= 9; ++ xx )
        {
        	placeBlock(world, Block.stairsWoodOak, ForgeDirection.NORTH, xx,4,2, bb);
        	placeBlock(world, Block.stairsWoodOak, ForgeDirection.NORTH, xx,5,3, bb);
        	placeBlock(world, Block.woodSingleSlab, 0, xx,6,4, bb);
        	placeBlock(world, Block.woodSingleSlab, 0, xx,6,6, bb);
        	placeBlock(world, Block.stairsWoodOak, ForgeDirection.SOUTH, xx,5,7, bb);
        	placeBlock(world, Block.stairsWoodOak, ForgeDirection.SOUTH, xx,4,8, bb);
        }
        placeBlock(world, Block.stairsWoodOak, Dir.getMeta(Block.stairsWoodOak, ForgeDirection.SOUTH, coordBaseMode) + 4, 1,4,3, bb);
        placeBlock(world, Block.stairsWoodOak, Dir.getMeta(Block.stairsWoodOak, ForgeDirection.SOUTH, coordBaseMode) + 4, 9,4,3, bb);
        placeBlock(world, Block.stairsWoodOak, Dir.getMeta(Block.stairsWoodOak, ForgeDirection.SOUTH, coordBaseMode) + 4, 1,5,4, bb);
        placeBlock(world, Block.stairsWoodOak, Dir.getMeta(Block.stairsWoodOak, ForgeDirection.SOUTH, coordBaseMode) + 4, 9,5,4, bb);
        placeBlock(world, Block.stairsWoodOak, Dir.getMeta(Block.stairsWoodOak, ForgeDirection.NORTH, coordBaseMode) + 4, 1,5,6, bb);
        placeBlock(world, Block.stairsWoodOak, Dir.getMeta(Block.stairsWoodOak, ForgeDirection.NORTH, coordBaseMode) + 4, 9,5,6, bb);
        placeBlock(world, Block.stairsWoodOak, Dir.getMeta(Block.stairsWoodOak, ForgeDirection.NORTH, coordBaseMode) + 4, 1,4,7, bb);
        placeBlock(world, Block.stairsWoodOak, Dir.getMeta(Block.stairsWoodOak, ForgeDirection.NORTH, coordBaseMode) + 4, 9,4,7, bb);

        // Decoration
        placeBlock(world, Block.hay, random.nextInt(3) * 4, 3,1,6, bb);
        placeBlock(world, Block.hay, random.nextInt(3) * 4, 7,1,6, bb);
        if( !hasMadeChest )
        {
        	int xx = (random.nextBoolean() ? 3 : 7);
        	int chestX = getXWithOffset(xx, 6);
        	int chestY = getYWithOffset(0);
        	int chestZ = getZWithOffset(xx, 6);
        	if( bb.isVecInside(chestX, chestY, chestZ) )
        	{
        		hasMadeChest = true;
                generateStructureChestContents(world, bb, random, xx,0,6,
                		ChestGenHooks.getItems(STALL_CHEST, random), ChestGenHooks.getCount(STALL_CHEST, random));
        	}
        }

        if( !horseSpawned )
        {
        	horseSpawned = (spawnEntity(world, bb, (random.nextBoolean() ? 4 : 6),1,5, 1) == 1);
        }
        spawnVillagers(world, bb, 5,1,1, 1);

        return true;
	}

	@Override
	protected int getVillagerType(int alreadySpawned)
	{
		return (Taverns.config.hostlerID > 0 ? Taverns.config.hostlerID : super.getVillagerType(alreadySpawned));
	}
	

	@Override
	protected EntityLivingBase getEntity(World world, int index)
	{
		return new EntityHorse(world);
	}

}
