package de.vernideas.mc.taverns.gen;

import static net.minecraftforge.common.ChestGenHooks.VILLAGE_BLACKSMITH;

import java.util.List;
import java.util.Random;

import cpw.mods.fml.common.FMLLog;
import net.minecraft.block.Block;
import net.minecraft.item.Item;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.WeightedRandomChestContent;
import net.minecraft.world.World;
import net.minecraft.world.gen.structure.ComponentVillage;
import net.minecraft.world.gen.structure.ComponentVillageStartPiece;
import net.minecraft.world.gen.structure.MapGenStructureIO;
import net.minecraft.world.gen.structure.StructureBoundingBox;
import net.minecraft.world.gen.structure.StructureComponent;
import net.minecraftforge.common.ChestGenHooks;
import net.minecraftforge.common.ForgeDirection;
import de.vernideas.mc.taverns.Settings;
import de.vernideas.mc.taverns.Taverns;
import de.vernideas.mc.taverns.direction.Dir;

public class ComponentVillageTavern extends ComponentVillageBase
{
    public static final String TAVERN_CHEST = "taverns:tavernChest";
    public static final WeightedRandomChestContent[] tavernChestContents = new WeightedRandomChestContent[]
    {
        new WeightedRandomChestContent(Item.diamond.itemID, 0, 1, 3, 1),
        new WeightedRandomChestContent(Item.ingotIron.itemID, 0, 1, 5, 10),
        new WeightedRandomChestContent(Item.bed.itemID, 0, 1, 3, 30),
        new WeightedRandomChestContent(Item.bread.itemID, 0, 1, 3, 15),
        new WeightedRandomChestContent(Item.appleRed.itemID, 0, 1, 3, 15),
        new WeightedRandomChestContent(Item.pickaxeIron.itemID, 0, 1, 1, 5),
        new WeightedRandomChestContent(Item.swordIron.itemID, 0, 1, 1, 5),
        new WeightedRandomChestContent(Item.plateIron.itemID, 0, 1, 1, 5),
        new WeightedRandomChestContent(Item.helmetIron.itemID, 0, 1, 1, 5),
        new WeightedRandomChestContent(Item.legsIron.itemID, 0, 1, 1, 5),
        new WeightedRandomChestContent(Item.bootsIron.itemID, 0, 1, 1, 5),
        new WeightedRandomChestContent(Block.obsidian.blockID, 0, 1, 7, 5),
        new WeightedRandomChestContent(Block.sapling.blockID, 0, 3, 7, 5)
    };
    
    public static boolean initialized = false;;
    
    public static void init(Settings config)
    {
    	if( !initialized )
    	{
    		initialized = true;
    		ChestGenHooks.getInfo(TAVERN_CHEST).setMin(1);
    		ChestGenHooks.getInfo(TAVERN_CHEST).setMax(10);
            for( int i = 0; i < tavernChestContents.length; ++ i )
            {
            	ChestGenHooks.addItem(TAVERN_CHEST, tavernChestContents[i]);
            }
            MapGenStructureIO.func_143031_a(ComponentVillageTavern.class, "taverns:ViT");
    	}
    }
    
    private static final int HEIGHT = 10;
    
    public static Object buildComponent(ComponentVillageStartPiece startPiece, List pieces, Random random,
    		int x, int y, int z, int direction, int type)
    {
    	StructureBoundingBox structBB = StructureBoundingBox.getComponentToAddBoundingBox(x, y, z, 0, 0, 0, 14, HEIGHT, 8, direction);
    	if( canVillageGoDeeper(structBB) )
    	{
            if( null == StructureComponent.findIntersecting(pieces, structBB) )
            {
                    return new ComponentVillageTavern(startPiece, type, random, structBB, direction);
            }
    	}
    	return null;
    }
    
    private int averageGroundLevel = -1;

    // Because Minecraft likes to call the generation algorithm half a dozen times at the same location ...
    private boolean hasMadeChest;
    private int carpetColor;
    private int flower;

    public ComponentVillageTavern() {}
    
	public ComponentVillageTavern(ComponentVillageStartPiece startPiece, int pieces, Random rnd, StructureBoundingBox structBB, int direction)
	{
		super(startPiece, pieces);
        coordBaseMode = direction;
        boundingBox = structBB;
        carpetColor = rnd.nextInt(16);
        flower = rnd.nextInt(16) - 4; // Negative numbers = no flowerpot
    }

	/** writeNBTTag */
    @Override
    protected void func_143012_a(NBTTagCompound par1NBTTagCompound)
    {
        super.func_143012_a(par1NBTTagCompound);
        par1NBTTagCompound.setBoolean("Chest", hasMadeChest);
        par1NBTTagCompound.setInteger("CarpetC", carpetColor);
        par1NBTTagCompound.setInteger("Flower", flower);
    }

	/** readNBTTag */
    @Override
    protected void func_143011_b(NBTTagCompound par1NBTTagCompound)
    {
        super.func_143011_b(par1NBTTagCompound);
        hasMadeChest = par1NBTTagCompound.getBoolean("Chest");
        carpetColor = par1NBTTagCompound.getInteger("CarpetC");
        flower = par1NBTTagCompound.getInteger("Flower");
    }

	@Override
	public boolean addComponentParts(World world, Random rnd, StructureBoundingBox bb)
	{
        if( averageGroundLevel < 0 )
        {
            averageGroundLevel = getAverageGroundLevel(world, bb);
            if (averageGroundLevel < 0)
            {
                return true;
            }
            boundingBox.offset(0, this.averageGroundLevel - boundingBox.maxY + HEIGHT - 2, 0);
        }

        // Clear area for the tavern building
        fillWithBlocks(world, bb, 0,0,0, 14,9,7, 0, 0, false);
        for( int xx = 0; xx < 14; ++ xx )
        {
            for( int zz = 0; zz < 8; ++ zz)
            {
                clearCurrentPositionBlocksUpwards(world, xx,0,zz, bb);
                fillDownwards(world, Block.cobblestone, 0, xx,-1,zz, bb);
            }
        }
                
        // floors
        fill(world, bb, 0,0,1, 14,0,7, Block.dirt, Block.dirt); // backside
        fill(world, bb, 0,0,0, 14,0,0, Block.gravel, Block.gravel); // front
        fill(world, bb, 1,0,1, 4,0,6, Block.cobblestone, Block.cobblestone); // main floor left
        fill(world, bb, 5,0,1, 12,0,6, Block.planks, Block.planks); // main floor right
        fill(world, bb, 1,4,1, 12,4,6, Block.planks, Block.planks); // upper floor
        fill(world, bb, 7,8,1, 12,8,6, Block.planks, Block.planks); // roof

        //// walls
        // wall - left side
        fill(world, bb, 1,1,1, 1,4,1, Block.wood, Block.wood);
        fill(world, bb, 1,1,2, 1,3,5, Block.cobblestone, Block.cobblestone);
        fill(world, bb, 1,1,6, 1,4,6, Block.wood, Block.wood);

        fill(world, bb, 2,3,1, 3,3,1, Block.planks, Block.planks);
        fill(world, bb, 2,1,1, 3,1,1, Block.cobblestone, Block.cobblestone);
        fill(world, bb, 2,2,1, 3,2,1, Block.thinGlass, Block.thinGlass);
        fill(world, bb, 4,1,1, 4,3,1, Block.wood, Block.wood);
        fill(world, bb, 5,3,1, 6,3,1, Block.wood, Block.wood);
        fill(world, bb, 7,1,1, 7,7,1, Block.wood, Block.wood);
        placeBlock(world, Block.wood, ForgeDirection.WEST, 4, 3, 1, bb);
        placeBlock(world, Block.wood, ForgeDirection.WEST, 7, 3, 1, bb);
        fill(world, bb, 8,1,1, 11,1,1, Block.cobblestone, Block.cobblestone);
        fill(world, bb, 8,2,1, 11,7,1, Block.planks, Block.planks);
        fill(world, bb, 8,2,1, 11,2,1, Block.thinGlass, Block.thinGlass);
        fill(world, bb, 12,1,1, 12,7,1, Block.wood, Block.wood);

        fill(world, bb, 12,1,2, 12,1,5, Block.cobblestone, Block.cobblestone);
        fill(world, bb, 12,2,2, 12,7,5, Block.planks, Block.planks);
        fill(world, bb, 12,2,3, 12,2,4, Block.thinGlass, Block.thinGlass);
        fill(world, bb, 12,6,3, 12,6,4, Block.thinGlass, Block.thinGlass);
        fill(world, bb, 12,1,6, 12,7,6, Block.wood, Block.wood);
        
        fill(world, bb, 7,1,6, 11,1,6, Block.cobblestone, Block.cobblestone);
        fill(world, bb, 7,2,6, 11,7,6, Block.planks, Block.planks);
        placeBlock(world, Block.thinGlass, 0, 10,6,6, bb);
        fill(world, bb, 6,1,6, 6,8,6, Block.wood, Block.wood);
        placeBlock(world, Block.wood, 0, 5,3,6, bb);
        fill(world, bb, 4,1,6, 4,3,6, Block.wood, Block.wood);
        fill(world, bb, 2,1,6, 3,3,6, Block.cobblestone, Block.cobblestone);

        fill(world, bb, 7,5,2, 7,7,5, Block.planks, Block.planks);
        fill(world, bb, 6,5,5, 6,8,5, Block.planks, Block.planks);
                
        // roofing
        for( int xx = 0; xx <= 13; ++ xx )
        {
			placeBlock(world, Block.stairsWoodOak, ForgeDirection.NORTH, xx, 4, 0, bb);
			placeBlock(world, Block.stairsWoodOak, ForgeDirection.SOUTH, xx, 4, 7, bb);
        }
        
        for( int zz = 0; zz <= 7; ++ zz )
        {
        	placeBlock(world, Block.stairsWoodOak, ForgeDirection.EAST, 0, 4, zz, bb);
            placeBlock(world, Block.stairsWoodOak, ForgeDirection.WEST, 13, 4, zz, bb);
        }
        
        for( int xx = 6; xx <= 13; ++ xx)
        {
        	placeBlock(world, Block.stairsWoodOak, ForgeDirection.NORTH, xx, 7, 0, bb);
        	placeBlock(world, Block.stairsWoodOak, ForgeDirection.SOUTH, xx, 7, 7, bb);
        	placeBlock(world, Block.stairsWoodOak, ForgeDirection.NORTH, xx, 8, 1, bb);
        	placeBlock(world, Block.stairsWoodOak, ForgeDirection.SOUTH, xx, 8, 6, bb);
        }
        
        for( int xx = 7; xx <= 13; ++ xx)
        {
        	placeBlock(world, Block.stairsWoodOak, ForgeDirection.NORTH, xx, 9, 2, bb);
        	placeBlock(world, Block.stairsWoodOak, ForgeDirection.SOUTH, xx, 9, 5, bb);
        }
        
        for( int zz = 2; zz <= 5; ++ zz)
        {
        	placeBlock(world, Block.stairsWoodOak, ForgeDirection.EAST, 6,8,zz, bb);
        }
        
        for( int zz = 3; zz <= 4; ++ zz )
        {
        	placeBlock(world, Block.stairsWoodOak, ForgeDirection.WEST, 13,9,zz, bb);
        	placeBlock(world, Block.stairsWoodOak, ForgeDirection.EAST, 12,9,zz, bb);        
        	placeBlock(world, Block.stairsWoodOak, ForgeDirection.EAST, 7,9,zz, bb);
        	placeBlock(world, Block.stairsWoodOak, ForgeDirection.WEST, 8,9,zz, bb);
        }
        
        for( int zz = 5; zz <= 7; ++ zz )
        {
        	placeBlock(world, Block.stairsWoodOak, ForgeDirection.EAST, 5,7,zz, bb);
        }
        
        fillWithAir(world, bb, 6,8,4, 6,8,4);
                                
        // Interior
        fill(world, bb, 1,5,6, 5,5,6, Block.fence, Block.fence);
        fill(world, bb, 1,5,1, 1,5,5, Block.fence, Block.fence);
        fill(world, bb, 2,5,1, 6,5,1, Block.fence, Block.fence);
                
        for( int yy = 5; yy <= 8; ++ yy )
        {
        	placeBlock(world, Block.ladder, ForgeDirection.WEST, 6,yy,4, bb);
        }

        placeBlock(world, Block.bookShelf, 0, 11,5,2, bb);
        placeBlock(world, Block.torchWood, 0, 11,6,2, bb);

        placeBlock(world, Block.bed, ForgeDirection.SOUTH, 10,5,3, bb);
        placeBlock(world, Block.bed, Dir.getMeta(Block.bed, ForgeDirection.SOUTH, coordBaseMode) + 8, 10,5,2, bb);
        
        if( !hasMadeChest )
        {
        	int chestX = getXWithOffset(9, 2);
        	int chestY = getYWithOffset(5);
        	int chestZ = getZWithOffset(9, 2);
        	if( bb.isVecInside(chestX, chestY, chestZ) )
        	{
        		hasMadeChest = true;
                generateStructureChestContents(world, bb, rnd, 9,5,2,
                		ChestGenHooks.getItems(TAVERN_CHEST, rnd), ChestGenHooks.getCount(TAVERN_CHEST, rnd));
        	}
        }
        
        placeDoor(world, bb, rnd, 7,5,2, ForgeDirection.WEST);
        placeDoor(world, bb, rnd, 6,1,1, ForgeDirection.SOUTH);
        placeDoor(world, bb, rnd, 5,1,1, ForgeDirection.SOUTH);
        placeDoor(world, bb, rnd, 5,1,6, ForgeDirection.NORTH);

        fill(world, bb, 4,1,5, 4,3,5, Block.wood, Block.wood);
        fill(world, bb, 7,1,2, 7,3,2, Block.wood, Block.wood);
                
        placeBlock(world, Block.stairsWoodOak, ForgeDirection.EAST, 7,1,5, bb);
        placeBlock(world, Block.stairsWoodOak, ForgeDirection.EAST, 8,2,5, bb);
        placeBlock(world, Block.stairsWoodOak, ForgeDirection.EAST, 9,3,5, bb);
        placeBlock(world, Block.stairsWoodOak, ForgeDirection.EAST, 10,4,5, bb);
        fillWithAir(world, bb, 7,4,5, 9,4,5);
                
        placeBlock(world, Block.planks, 0, 8,1,5, bb);
        placeBlock(world, Block.stairsWoodSpruce, ForgeDirection.NORTH, 9,1,5, bb);
        placeBlock(world, Block.stairsWoodSpruce, ForgeDirection.NORTH, 10,1,5, bb);
        placeBlock(world, Block.stairsWoodSpruce, ForgeDirection.NORTH, 11,1,5, bb);
        placeBlock(world, Block.stairsWoodSpruce, ForgeDirection.EAST, 11,1,4, bb);
        placeBlock(world, Block.stairsWoodSpruce, ForgeDirection.EAST, 11,1,3, bb);
        placeBlock(world, Block.planks, 0, 11,1,2, bb);
        placeBlock(world, Block.stairsWoodOak, ForgeDirection.EAST, 10,1,2, bb);
        placeBlock(world, Block.fence, 0, 9,1,2, bb);
        placeBlock(world, Block.pressurePlatePlanks, 0, 9,2,2, bb);
        placeBlock(world, Block.stairsWoodOak, ForgeDirection.WEST, 8,1,2, bb);
                
        fill(world, bb, 4,1,2, 4,1,3, Block.planks, Block.planks);
        fill(world, bb, 2,1,2, 2,1,3, Block.planks, Block.planks);

        placeBlock(world, Block.brewingStand, 0, 2,2,3, bb);
        placeBlock(world, Block.cauldron, rnd.nextInt(3) + 1, 3,1,5, bb);
        placeBlock(world, Block.trapdoor, ForgeDirection.NORTH, 4, 2, 4, bb);
        
        // Furnace sets it rotation based on neighbours - we need to enforce our direction instead, since the neighbours might
        // be in a different chunk and thus not loaded yet.
        placeBlock(world, Block.furnaceIdle, 0, 2,1,5, bb);
        setMetadata(world, Dir.getMeta(Block.furnaceIdle, ForgeDirection.SOUTH, coordBaseMode), 2,1,5, bb);

        // Carpets
        placeBlock(world, Block.carpet, carpetColor,  8,1,3, bb);
        placeBlock(world, Block.carpet, carpetColor,  8,1,4, bb);
        placeBlock(world, Block.carpet, carpetColor,  9,1,3, bb);
        placeBlock(world, Block.carpet, carpetColor,  9,1,4, bb);
        placeBlock(world, Block.carpet, carpetColor, 10,1,3, bb);
        placeBlock(world, Block.carpet, carpetColor, 10,1,4, bb);
        
        if( flower >= 0 )
        {
        	placeBlock(world, Block.flowerPot, flower, 6,6,1, bb);
        }
        
        // Torches - wall-mounted torches can "fall off" if the wall happens to be in another chunk. Let's try to prevent it
        // by putting something below them for the duration of the generation, as well as setting the direction
        // of the torch. This auto-alignment is also a problem for doors, but it's less visible there.
        placeBlock(world, Block.torchWood, 0, 1,6,6, bb);
        placeBlock(world, Block.torchWood, 0, 1,6,1, bb);
        placeBlock(world, Block.torchWood, 0, 9,9,4, bb);
        placeBlock(world, Block.torchWood, 0, 4,3,0, bb);
        placeBlock(world, Block.torchWood, 0, 7,3,0, bb);
        placeBlock(world, Block.torchWood, 0, 5,3,7, bb);
        placeBlock(world, Block.torchWood, 0, 2,3,5, bb);
        
        placeBlock(world, Block.stone, 0, 4,2,2, bb);
        placeBlock(world, Block.torchWood, ForgeDirection.NORTH, 4,3,2, bb);
        placeAir(world, 4,2,2, bb);

        placeBlock(world, Block.stone, 0, 9,2,2, bb);
        placeBlock(world, Block.torchWood, ForgeDirection.NORTH, 9,3,2, bb);
        placeBlock(world, Block.pressurePlatePlanks, 0, 9,2,2, bb);
        
        spawnVillagers(world, bb, 2,1,2, 1);
        
        return true;
	}
	
	@Override
	protected int getVillagerType(int alreadySpawned)
	{
		return (Taverns.config.barWenchID > 0 ? Taverns.config.barWenchID : super.getVillagerType(alreadySpawned));
	}
}
